@@ -1,8 +1,10 @@
 # -*- coding: utf-8 -*-
 """Dataset class containing all logic for creating, checking, and updating datasets and associated resources.
 """
+import fnmatch
 import logging
 import sys
+from copy import deepcopy
 from datetime import datetime
 from os.path import join
 from typing import List, Union, Optional, Dict, Any
@@ -17,10 +19,11 @@ from six.moves import range
 import hdx.data.organization
 import hdx.data.resource
 import hdx.data.showcase
+import hdx.data.user
 from hdx.data.hdxobject import HDXObject, HDXError
-from hdx.data.user import User
 from hdx.hdx_configuration import Configuration
 from hdx.hdx_locations import Locations
+from hdx.hdx_tagscleanup import Tags
 
 logger = logging.getLogger(__name__)
 
@@ -73,7 +76,7 @@ class Dataset(HDXObject):
     }
 
     def __init__(self, initial_data=None, configuration=None):
-        # type: (Optional[Dict], Optional[bool], Optional[Configuration]) -> None
+        # type: (Optional[Dict], Optional[Configuration]) -> None
         if not initial_data:
             initial_data = dict()
         super(Dataset, self).__init__(dict(), configuration=configuration)
@@ -95,11 +98,13 @@ class Dataset(HDXObject):
             'show': 'package_show',
             'update': 'package_update',
             'create': 'package_create',
-            'delete': 'package_delete',
+            'delete': 'hdx_dataset_purge',
             'search': 'package_search',
+            'reorder': 'package_resource_reorder',
             'list': 'package_list',
             'all': 'current_package_list_with_resources',
-            'hxl': 'package_hxl_update'
+            'hxl': 'package_hxl_update',
+            'create_default_views': 'package_create_default_resource_views'
         }
 
     def __setitem__(self, key, value):
@@ -137,6 +142,26 @@ class Dataset(HDXObject):
         self.resources = list()
         """:type : List[hdx.data.resource.Resource]"""
 
+    def _get_resource_from_obj(self, resource):
+        # type: (Union[hdx.data.resource.Resource,Dict,str]) -> hdx.data.resource.Resource
+        """Add new or update existing resource in dataset with new metadata
+
+        Args:
+            resource (Union[hdx.data.resource.Resource,Dict,str]): Either resource id or resource metadata from a Resource object or a dictionary
+
+        Returns:
+            hdx.data.resource.Resource: Resource object
+        """
+        if isinstance(resource, str):
+            if is_valid_uuid(resource) is False:
+                raise HDXError('%s is not a valid resource id!' % resource)
+            resource = hdx.data.resource.Resource.read_from_hdx(resource, configuration=self.configuration)
+        elif isinstance(resource, dict):
+            resource = hdx.data.resource.Resource(resource, configuration=self.configuration)
+        if not isinstance(resource, hdx.data.resource.Resource):
+            raise HDXError('Type %s cannot be added as a resource!' % type(resource).__name__)
+        return resource
+
     def add_update_resource(self, resource, ignore_datasetid=False):
         # type: (Union[hdx.data.resource.Resource,Dict,str], bool) -> None
         """Add new or update existing resource in dataset with new metadata
@@ -148,28 +173,20 @@ class Dataset(HDXObject):
         Returns:
             None
         """
-        if isinstance(resource, str):
-            if is_valid_uuid(resource) is False:
-                raise HDXError('%s is not a valid resource id!' % resource)
-            resource = hdx.data.resource.Resource.read_from_hdx(resource, configuration=self.configuration)
-        elif isinstance(resource, dict):
-            resource = hdx.data.resource.Resource(resource, configuration=self.configuration)
-        if isinstance(resource, hdx.data.resource.Resource):
-            if 'package_id' in resource:
-                if not ignore_datasetid:
-                    raise HDXError('Resource %s being added already has a dataset id!' % (resource['name']))
-            resource_updated = self._addupdate_hdxobject(self.resources, 'name', resource)
-            resource_updated.set_file_to_upload(resource.get_file_to_upload())
-            return
-        raise HDXError('Type %s cannot be added as a resource!' % type(resource).__name__)
+        resource = self._get_resource_from_obj(resource)
+        if 'package_id' in resource:
+            if not ignore_datasetid:
+                raise HDXError('Resource %s being added already has a dataset id!' % (resource['name']))
+        resource_updated = self._addupdate_hdxobject(self.resources, 'name', resource)
+        resource_updated.set_file_to_upload(resource.get_file_to_upload())
 
     def add_update_resources(self, resources, ignore_datasetid=False):
-        # type: (List[Union[hdx.data.resource.Resource,Dict,str]]) -> None
+        # type: (List[Union[hdx.data.resource.Resource,Dict,str]], bool) -> None
         """Add new or update existing resources with new metadata to the dataset
 
         Args:
             resources (List[Union[hdx.data.resource.Resource,Dict,str]]): A list of either resource ids or resources metadata from either Resource objects or dictionaries
-            ignore_datasetid (Optional[bool]): Whether to ignore dataset id in the resource
+            ignore_datasetid (bool): Whether to ignore dataset id in the resource. Defaults to False.
 
         Returns:
             None
@@ -216,6 +233,29 @@ class Dataset(HDXObject):
         """
         return self.resources[index]
 
+    def reorder_resources(self, resource_ids, hxl_update=True):
+        # type: (List[str], bool) -> None
+        """Reorder resources in dataset according to provided list.
+        If only some resource ids are supplied then these are
+        assumed to be first and the other resources will stay in
+        their original order.
+
+        Args:
+            resource_ids (List[str]): List of resource ids
+            hxl_update (bool): Whether to call package_hxl_update. Defaults to True.
+
+        Returns:
+            None
+        """
+        dataset_id = self.data.get('id')
+        if not dataset_id:
+            raise HDXError('Dataset has no id! It must be read, created or updated first.')
+        data = {'id': dataset_id,
+                'order': resource_ids}
+        self._write_to_hdx('reorder', data, 'package_id')
+        if hxl_update:
+            self.hxl_update()
+
     def update_from_yaml(self, path=join('config', 'hdx_dataset_static.yml')):
         # type: (str) -> None
         """Update dataset metadata with static metadata from YAML file
@@ -288,13 +328,13 @@ class Dataset(HDXObject):
         return True
 
     def check_required_fields(self, ignore_fields=list(), allow_no_resources=False):
-        # type: (List[str], Optional[bool]) -> None
+        # type: (List[str], bool) -> None
         """Check that metadata for dataset and its resources is complete. The parameter ignore_fields
         should be set if required to any fields that should be ignored for the particular operation.
 
         Args:
             ignore_fields (List[str]): Fields to ignore. Default is [].
-            allow_no_resources (Optional[bool]): Whether to allow no resources. Defaults to False.
+            allow_no_resources (bool): Whether to allow no resources. Defaults to False.
 
         Returns:
             None
@@ -309,83 +349,161 @@ class Dataset(HDXObject):
                 ignore_fields = ['package_id']
                 resource.check_required_fields(ignore_fields=ignore_fields)
 
-    def _dataset_merge_hdx_update(self, update_resources, update_resources_by_name):
-        # type: (bool) -> None
+    def _dataset_merge_filestore_resource(self, resource, updated_resource, filestore_resources, ignore_fields):
+        # type: (hdx.data.Resource, hdx.data.Resource, List[hdx.data.Resource], List[str]) -> None
+        """Helper method to merge updated resource from dataset into HDX resource read from HDX including filestore.
+
+        Args:
+            resource (hdx.data.Resource): Resource read from HDX
+            updated_resource (hdx.data.Resource): Updated resource from dataset
+            filestore_resources (List[hdx.data.Resource]): List of resources that use filestore (to be appended to)
+            ignore_fields (List[str]): List of fields to ignore when checking resource
+
+        Returns:
+            None
+        """
+        merge_two_dictionaries(resource, updated_resource)
+        if updated_resource.get_file_to_upload():
+            resource.set_file_to_upload(updated_resource.get_file_to_upload())
+            filestore_resources.append(resource)
+            if 'url' not in resource:
+                resource['url'] = 'ignore'
+        resource.check_required_fields(ignore_fields=ignore_fields)
+
+    def _dataset_merge_filestore_newresource(self, new_resource, ignore_fields, filestore_resources):
+        # type: (hdx.data.Resource, List[str], List[hdx.data.Resource]) -> None
+        """Helper method to add new resource from dataset including filestore.
+
+        Args:
+            new_resource (hdx.data.Resource): New resource from dataset
+            ignore_fields (List[str]): List of fields to ignore when checking resource
+            filestore_resources (List[hdx.data.Resource]): List of resources that use filestore (to be appended to)
+
+        Returns:
+            None
+        """
+        new_resource.check_required_fields(ignore_fields=ignore_fields)
+        self.resources.append(new_resource)
+        if new_resource.get_file_to_upload():
+            filestore_resources.append(new_resource)
+            if 'url' not in new_resource:
+                new_resource['url'] = 'ignore'
+
+    def _add_filestore_resources(self, filestore_resources, create_default_views, hxl_update):
+        # type: (List[hdx.data.Resource], bool) -> None
+        """Helper method to create files in filestore by updating resources.
+
+        Args:
+            filestore_resources (List[hdx.data.Resource]): List of resources that use filestore (to be appended to)
+            create_default_views (bool): Whether to call package_create_default_resource_views.
+            hxl_update (bool): Whether to call package_hxl_update.
+
+        Returns:
+            None
+        """
+        for resource in filestore_resources:
+            for created_resource in self.data['resources']:
+                if resource['name'] == created_resource['name']:
+                    merge_two_dictionaries(resource.data, created_resource)
+                    del resource['url']
+                    resource.update_in_hdx()
+                    merge_two_dictionaries(created_resource, resource.data)
+                    break
+        self.init_resources()
+        self.separate_resources()
+        if create_default_views:
+            self.create_default_views()
+        if hxl_update:
+            self.hxl_update()
+
+    def _dataset_merge_hdx_update(self, update_resources, update_resources_by_name,
+                                  remove_additional_resources, create_default_views, hxl_update):
+        # type: (bool, bool, bool, bool) -> None
         """Helper method to check if dataset or its resources exist and update them
 
         Args:
             update_resources (bool): Whether to update resources
             update_resources_by_name (bool): Compare resource names rather than position in list
+            remove_additional_resources (bool): Remove additional resources found in dataset (if updating)
+            create_default_views (bool): Whether to call package_create_default_resource_views.
+            hxl_update (bool): Whether to call package_hxl_update.
 
         Returns:
             None
         """
+        # 'old_data' here is the data we want to use for updating while 'data' is the data read from HDX
         merge_two_dictionaries(self.data, self.old_data)
         if 'resources' in self.data:
             del self.data['resources']
-        old_resources = self.old_data.get('resources', None)
+        updated_resources = self.old_data.get('resources', None)
         filestore_resources = list()
-        if update_resources and old_resources:
+        if update_resources and updated_resources:
             ignore_fields = ['package_id']
             if update_resources_by_name:
                 resource_names = set()
                 for resource in self.resources:
                     resource_name = resource['name']
                     resource_names.add(resource_name)
-                    for old_resource in old_resources:
-                        if resource_name == old_resource['name']:
+                    for updated_resource in updated_resources:
+                        if resource_name == updated_resource['name']:
                             logger.warning('Resource exists. Updating %s' % resource_name)
-                            merge_two_dictionaries(resource, old_resource)
-                            if old_resource.get_file_to_upload():
-                                resource.set_file_to_upload(old_resource.get_file_to_upload())
-                                filestore_resources.append(resource)
-                            resource.check_required_fields(ignore_fields=ignore_fields)
+                            self._dataset_merge_filestore_resource(resource, updated_resource,
+                                                                   filestore_resources, ignore_fields)
                             break
-                for old_resource in old_resources:
-                    if not old_resource['name'] in resource_names:
-                        old_resource.check_required_fields(ignore_fields=ignore_fields)
-                        self.resources.append(old_resource)
-                        if old_resource.get_file_to_upload():
-                            filestore_resources.append(old_resource)
+                updated_resource_names = set()
+                for updated_resource in updated_resources:
+                    updated_resource_name = updated_resource['name']
+                    updated_resource_names.add(updated_resource_name)
+                    if not updated_resource_name in resource_names:
+                        self._dataset_merge_filestore_newresource(updated_resource, ignore_fields, filestore_resources)
+                if remove_additional_resources:
+                    resources_to_delete = list()
+                    for i, resource in enumerate(self.resources):
+                        resource_name = resource['name']
+                        if resource_name not in updated_resource_names:
+                            logger.warning('Removing additional resource %s!' % resource_name)
+                            resources_to_delete.append(i)
+                    for i in sorted(resources_to_delete, reverse=True):
+                        del self.resources[i]
+
             else:  # update resources by position
-                for i, old_resource in enumerate(old_resources):
+                for i, updated_resource in enumerate(updated_resources):
                     if len(self.resources) > i:
-                        old_resource_name = old_resource['name']
+                        updated_resource_name = updated_resource['name']
                         resource = self.resources[i]
                         resource_name = resource['name']
                         logger.warning('Resource exists. Updating %s' % resource_name)
-                        if resource_name != old_resource_name:
-                            logger.warning('Changing resource name to: %s' % old_resource_name)
-                        merge_two_dictionaries(resource, old_resource)
-                        if old_resource.get_file_to_upload():
-                            resource.set_file_to_upload(old_resource.get_file_to_upload())
-                            filestore_resources.append(resource)
-                        resource.check_required_fields(ignore_fields=ignore_fields)
+                        if resource_name != updated_resource_name:
+                            logger.warning('Changing resource name to: %s' % updated_resource_name)
+                        self._dataset_merge_filestore_resource(resource, updated_resource,
+                                                               filestore_resources, ignore_fields)
                     else:
-                        old_resource.check_required_fields(ignore_fields=ignore_fields)
-                        self.resources.append(old_resource)
-                        if old_resource.get_file_to_upload():
-                            filestore_resources.append(old_resource)
+                        self._dataset_merge_filestore_newresource(updated_resource, ignore_fields, filestore_resources)
+                if remove_additional_resources:
+                    resources_to_delete = list()
+                    for i, resource in enumerate(self.resources):
+                        if len(updated_resources) <= i:
+                            logger.warning('Removing additional resource %s!' % resource['name'])
+                            resources_to_delete.append(i)
+                    for i in sorted(resources_to_delete, reverse=True):
+                        del self.resources[i]
 
         if self.resources:
             self.data['resources'] = self._convert_hdxobjects(self.resources)
         self._save_to_hdx('update', 'id')
+        self._add_filestore_resources(filestore_resources, create_default_views, hxl_update)
 
-        for resource in filestore_resources:
-            for created_resource in self.data['resources']:
-                if resource['name'] == created_resource['name']:
-                    merge_two_dictionaries(resource.data, created_resource)
-                    resource.update_in_hdx()
-                    merge_two_dictionaries(created_resource, resource.data)
-                    break
-
-    def update_in_hdx(self, update_resources=True, update_resources_by_name=True):
-        # type: (Optional[bool]) -> None
+    def update_in_hdx(self, update_resources=True, update_resources_by_name=True,
+                      remove_additional_resources=False, create_default_views=True, hxl_update=True):
+        # type: (bool, bool, bool, bool) -> None
         """Check if dataset exists in HDX and if so, update it
 
         Args:
-            update_resources (Optional[bool]): Whether to update resources. Defaults to True.
-            update_resources_by_name (Optional[bool]): Compare resource names rather than position in list. Defaults to True.
+            update_resources (bool): Whether to update resources. Defaults to True.
+            update_resources_by_name (bool): Compare resource names rather than position in list. Defaults to True.
+            remove_additional_resources (bool): Remove additional resources found in dataset. Defaults to False.
+            create_default_views (bool): Whether to call package_create_default_resource_views. Defaults to True.
+            hxl_update (bool): Whether to call package_hxl_update. Defaults to True.
 
         Returns:
             None
@@ -402,17 +520,23 @@ class Dataset(HDXObject):
             if not self._dataset_load_from_hdx(self.data['name']):
                 raise HDXError('No existing dataset to update!')
         self._dataset_merge_hdx_update(update_resources=update_resources,
-                                       update_resources_by_name=update_resources_by_name)
-        self.hxl_update()
+                                       update_resources_by_name=update_resources_by_name,
+                                       remove_additional_resources=remove_additional_resources,
+                                       create_default_views=create_default_views,
+                                       hxl_update=hxl_update)
 
-    def create_in_hdx(self, allow_no_resources=False, update_resources=True, update_resources_by_name=True):
-        # type: (Optional[bool]) -> None
+    def create_in_hdx(self, allow_no_resources=False, update_resources=True, update_resources_by_name=True,
+                      remove_additional_resources=False, create_default_views=True, hxl_update=True):
+        # type: (bool, bool, bool, bool, bool) -> None
         """Check if dataset exists in HDX and if so, update it, otherwise create it
 
         Args:
-            allow_no_resources (Optional[bool]): Whether to allow no resources. Defaults to False.
-            update_resources (Optional[bool]): Whether to update resources (if updating). Defaults to True.
-            update_resources_by_name (Optional[bool]): Compare resource names rather than position in list. Defaults to True.
+            allow_no_resources (bool): Whether to allow no resources. Defaults to False.
+            update_resources (bool): Whether to update resources (if updating). Defaults to True.
+            update_resources_by_name (bool): Compare resource names rather than position in list. Defaults to True.
+            remove_additional_resources (bool): Remove additional resources found in dataset (if updating). Defaults to False.
+            create_default_views (bool): Whether to call package_create_default_resource_views (if updating). Defaults to True.
+            hxl_update (bool): Whether to call package_hxl_update. Defaults to True.
 
         Returns:
             None
@@ -430,7 +554,10 @@ class Dataset(HDXObject):
         if loadedid:
             logger.warning('Dataset exists. Updating %s' % loadedid)
             self._dataset_merge_hdx_update(update_resources=update_resources,
-                                           update_resources_by_name=update_resources_by_name)
+                                           update_resources_by_name=update_resources_by_name,
+                                           remove_additional_resources=remove_additional_resources,
+                                           create_default_views=create_default_views,
+                                           hxl_update=hxl_update)
             return
 
         filestore_resources = list()
@@ -440,18 +567,11 @@ class Dataset(HDXObject):
                 resource.check_required_fields(ignore_fields=ignore_fields)
                 if resource.get_file_to_upload():
                     filestore_resources.append(resource)
+                    if 'url' not in resource:
+                        resource['url'] = 'ignore'
             self.data['resources'] = self._convert_hdxobjects(self.resources)
         self._save_to_hdx('create', 'name')
-        for resource in filestore_resources:
-            for created_resource in self.data['resources']:
-                if resource['name'] == created_resource['name']:
-                    merge_two_dictionaries(resource.data, created_resource)
-                    resource.update_in_hdx()
-                    merge_two_dictionaries(created_resource, resource.data)
-                    break
-        self.init_resources()
-        self.separate_resources()
-        self.hxl_update()
+        self._add_filestore_resources(filestore_resources, False, hxl_update)
 
     def delete_from_hdx(self):
         # type: () -> None
@@ -560,13 +680,14 @@ class Dataset(HDXObject):
         return dataset._write_to_hdx('list', kwargs, 'id')
 
     @classmethod
-    def get_all_datasets(cls, configuration=None, page_size=1000, **kwargs):
-        # type: (Optional[Configuration], Any) -> List['Dataset']
+    def get_all_datasets(cls, configuration=None, page_size=1000, check_duplicates=True, **kwargs):
+        # type: (Optional[Configuration], int, bool, Any) -> List['Dataset']
         """Get all datasets in HDX
 
         Args:
             configuration (Optional[Configuration]): HDX configuration. Defaults to global configuration.
             page_size (int): Size of page to return. Defaults to 1000.
+            check_duplicates (bool): Whether to check for duplicate datasets. Defaults to True.
             **kwargs: See below
             limit (int): Number of rows to return. Defaults to all datasets (sys.maxsize)
             offset (int): Offset in the complete result for where the set of returned datasets should begin
@@ -591,8 +712,11 @@ class Dataset(HDXObject):
                 kwargs['limit'] = rows
                 result = dataset._write_to_hdx('all', kwargs, 'id')
                 datasets = list()
-                if result:
+                if isinstance(result, list):
                     no_results = len(result)
+                    if no_results == 0 and page == 0:
+                        all_datasets = None
+                        break
                     for datasetdict in result:
                         dataset = Dataset(configuration=configuration)
                         dataset.old_data = dict()
@@ -604,17 +728,20 @@ class Dataset(HDXObject):
                         break
                 else:
                     logger.debug(result)
-            names_list = [dataset['name'] for dataset in all_datasets]
-            names = set(names_list)
-            if len(names_list) != len(names):  # check for duplicates (shouldn't happen)
-                all_datasets = None
+            if all_datasets is None:
                 attempts += 1
-            # This check is no longer valid because of showcases being returned by package_list!
-            # elif total_rows == max_int:
-            #     all_names = set(Dataset.get_all_dataset_names())  # check dataset names match package_list
-            #     if names != all_names:
-            #         all_datasets = None
-            #         attempts += 1
+            elif check_duplicates:
+                names_list = [dataset['name'] for dataset in all_datasets]
+                names = set(names_list)
+                if len(names_list) != len(names):  # check for duplicates (shouldn't happen)
+                    all_datasets = None
+                    attempts += 1
+                # This check is no longer valid because of showcases being returned by package_list!
+                # elif total_rows == max_int:
+                #     all_names = set(Dataset.get_all_dataset_names())  # check dataset names match package_list
+                #     if names != all_names:
+                #         all_datasets = None
+                #         attempts += 1
         if attempts == cls.max_attempts and all_datasets is None:
             raise HDXError('Maximum attempts reached for getting all datasets!')
         return all_datasets
@@ -893,7 +1020,7 @@ class Dataset(HDXObject):
             tags (List[str]): list of tags to add
 
         Returns:
-            bool: True if all tags added or False if any already present.
+            bool: True if all tags added or False if any already present
         """
         return self._add_tags(tags)
 
@@ -909,6 +1036,30 @@ class Dataset(HDXObject):
         """
         return self._remove_hdxobject(self.data.get('tags'), tag, matchon='name')
 
+    def is_subnational(self):
+        # type: () -> bool
+        """Return if the dataset is subnational
+
+        Returns:
+            bool: True if the dataset is subnational, False if not
+        """
+        return self.data['subnational'] == '1'
+
+    def set_subnational(self, subnational):
+        # type: (bool) -> None
+        """Set if dataset is subnational or national
+
+        Args:
+            subnational (bool): True for subnational, False for national
+
+        Returns:
+            None
+        """
+        if subnational:
+            self.data['subnational'] = '1'
+        else:
+            self.data['subnational'] = '0'
+
     def get_location(self, locations=None):
         # type: (Optional[List[str]]) -> List[str]
         """Return the dataset's location
@@ -926,13 +1077,13 @@ class Dataset(HDXObject):
                                                      configuration=self.configuration) for x in countries]
 
     def add_country_location(self, country, exact=True, locations=None, use_live=True):
-        # type: (str, Optional[bool],Optional[List[str]]) -> bool
+        # type: (str, bool,Optional[List[str]]) -> bool
         """Add a country. If an iso 3 code is not provided, value is parsed and if it is a valid country name,
         converted to an iso 3 code. If the country is already added, it is ignored.
 
         Args:
             country (str): Country to add
-            exact (Optional[bool]): True for exact matching or False to allow fuzzy matching. Defaults to True.
+            exact (bool): True for exact matching or False to allow fuzzy matching. Defaults to True.
             locations (Optional[List[str]]): Valid locations list. Defaults to list downloaded from HDX.
             use_live (bool): Try to get use latest country data from web rather than file in package. Defaults to True.
 
@@ -983,13 +1134,13 @@ class Dataset(HDXObject):
                                                                           use_live=use_live), locations=locations)
 
     def add_other_location(self, location, exact=True, alterror=None, locations=None):
-        # type: (str, Optional[bool], Optional[str], Optional[List[str]]) -> bool
+        # type: (str, bool, Optional[str], Optional[List[str]]) -> bool
         """Add a location which is not a country or region. Value is parsed and compared to existing locations in
         HDX. If the location is already added, it is ignored.
 
         Args:
             location (str): Location to add
-            exact (Optional[bool]): True for exact matching or False to allow fuzzy matching. Defaults to True.
+            exact (bool): True for exact matching or False to allow fuzzy matching. Defaults to True.
             alterror (Optional[str]): Alternative error message to builtin if location not found. Defaults to None.
             locations (Optional[List[str]]): Valid locations list. Defaults to list downloaded from HDX.
 
@@ -1032,16 +1183,16 @@ class Dataset(HDXObject):
         return res
 
     def get_maintainer(self):
-        # type: () -> User
+        # type: () -> hdx.data.user.User
         """Get the dataset's maintainer.
 
          Returns:
              User: Dataset's maintainer
         """
-        return User.read_from_hdx(self.data['maintainer'], configuration=self.configuration)
+        return hdx.data.user.User.read_from_hdx(self.data['maintainer'], configuration=self.configuration)
 
     def set_maintainer(self, maintainer):
-        # type: (Union[User,Dict,str]) -> None
+        # type: (Union[hdx.data.user.User,Dict,str]) -> None
         """Set the dataset's maintainer.
 
          Args:
@@ -1049,9 +1200,9 @@ class Dataset(HDXObject):
          Returns:
              None
         """
-        if isinstance(maintainer, User) or isinstance(maintainer, dict):
+        if isinstance(maintainer, hdx.data.user.User) or isinstance(maintainer, dict):
             if 'id' not in maintainer:
-                maintainer = User.read_from_hdx(maintainer['name'], configuration=self.configuration)
+                maintainer = hdx.data.user.User.read_from_hdx(maintainer['name'], configuration=self.configuration)
             maintainer = maintainer['id']
         elif not isinstance(maintainer, str):
             raise HDXError('Type %s cannot be added as a maintainer!' % type(maintainer).__name__)
@@ -1187,11 +1338,11 @@ class Dataset(HDXObject):
         return self.data.get('is_requestdata_type', False)
 
     def set_requestable(self, requestable=True):
-        # type: (Optional[bool]) -> None
+        # type: (bool) -> None
         """Set the dataset to be of type requestable or not
 
         Args:
-            requestable (Optional[bool]): Set whether dataset is requestable. Defaults to True.
+            requestable (bool): Set whether dataset is requestable. Defaults to True.
 
         Returns:
             None
@@ -1305,3 +1456,147 @@ class Dataset(HDXObject):
         if not self.is_requestable():
             raise NotRequestableError('remove_filetype is only applicable to requestable datasets!')
         return self._remove_string_from_commastring('file_types', filetype)
+
+    def clean_dataset_tags(self):
+        # type: (Optional[str], int) -> Tuple[bool, bool]
+        """Clean dataset tags according to tags cleanup spreadsheet and return if any changes occurred
+
+        Returns:
+            Tuple[bool, bool]: Returns (True if tags changed or False if not, True if error or False if not)
+        """
+        tags_dict, wildcard_tags = Tags.tagscleanupdicts()
+
+        def delete_tag(tag):
+            logger.info('%s - Deleting tag %s!' % (self.data['name'], tag))
+            return self.remove_tag(tag), False
+
+        def update_tag(tag, final_tags, wording, remove_existing=True):
+            text = '%s - %s: %s -> ' % (self.data['name'], wording, tag)
+            if not final_tags:
+                logger.error('%snothing!' % text)
+                return False, True
+            tags_lower_five = final_tags[:5].lower()
+            if tags_lower_five == 'merge' or tags_lower_five == 'split' or (
+                    ';' not in final_tags and len(final_tags) > 50):
+                logger.error('%s%s - Invalid final tag!' % (text, final_tags))
+                return False, True
+            if remove_existing:
+                self.remove_tag(tag)
+            tags = ', '.join(self.get_tags())
+            if self.add_tags(final_tags.split(';')):
+                logger.info('%s%s! Dataset tags: %s' % (text, final_tags, tags))
+            else:
+                logger.warning(
+                    '%s%s - At least one of the tags already exists! Dataset tags: %s' % (text, final_tags, tags))
+            return True, False
+
+        def do_action(tag, tags_dict_key):
+            whattodo = tags_dict[tags_dict_key]
+            action = whattodo[u'action']
+            final_tags = whattodo[u'final tags (semicolon separated)']
+            if action == u'Delete':
+                changed, error = delete_tag(tag)
+            elif action == u'Merge':
+                changed, error = update_tag(tag, final_tags, 'Merging')
+            elif action == u'Fix spelling':
+                changed, error = update_tag(tag, final_tags, 'Fixing spelling')
+            elif action == u'Non English':
+                changed, error = update_tag(tag, final_tags, 'Anglicising', remove_existing=False)
+            else:
+                changed = False
+                error = False
+            return changed, error
+
+        def process_tag(tag):
+            changed = False
+            error = False
+            if tag in tags_dict.keys():
+                changed, error = do_action(tag, tag)
+            else:
+                for wildcard_tag in wildcard_tags:
+                    if fnmatch.fnmatch(tag, wildcard_tag):
+                        changed, error = do_action(tag, wildcard_tag)
+                        break
+            return changed, error
+
+        anychange = False
+        anyerror = False
+        for tag in self.get_tags():
+            changed, error = process_tag(tag)
+            if changed:
+                anychange = True
+            if error:
+                anyerror = True
+
+        return anychange, anyerror
+
+    def preview_off(self):
+        # type: () -> None
+        """Set dataset preview off
+
+        Returns:
+            None
+        """
+        self.data['dataset_preview'] = 'no_preview'
+        for resource in self.resources:
+            resource.disable_dataset_preview()
+
+    def preview_resource(self):
+        # type: () -> None
+        """Set dataset preview on for an unspecified resource
+
+        Returns:
+            None
+        """
+        self.data['dataset_preview'] = 'resource_id'
+
+    def set_quickchart_resource(self, resource):
+        # type: (Union[hdx.data.resource.Resource,Dict,str,int]) -> bool
+        """Set the resource that will be used for displaying QuickCharts in dataset preview
+
+        Args:
+            resource (Union[hdx.data.resource.Resource,Dict,str,int]): Either resource id or name, resource metadata from a Resource object or a dictionary or position
+
+        Returns:
+            bool: Returns True if resource for QuickCharts in dataset preview set or False if not
+        """
+        if isinstance(resource, int) and not isinstance(resource, bool):
+            resource = self.get_resources()[resource]
+        if isinstance(resource, hdx.data.resource.Resource) or isinstance(resource, dict):
+            res = resource.get('id')
+            if res is None:
+                resource = resource['name']
+            else:
+                resource = res
+        elif not isinstance(resource, str):
+            raise hdx.data.hdxobject.HDXError('Resource id cannot be found in type %s!' % type(resource).__name__)
+        if is_valid_uuid(resource) is True:
+            search = 'id'
+        else:
+            search = 'name'
+        changed = False
+        for dataset_resource in self.resources:
+            if dataset_resource[search] == resource:
+                dataset_resource.enable_dataset_preview()
+                self.preview_resource()
+                changed = True
+            else:
+                dataset_resource.disable_dataset_preview()
+        return changed
+
+    def create_default_views(self, create_datastore_views=False):
+        # type: (bool) -> None
+        """Create default resource views for all resources in dataset
+
+        Args:
+            create_datastore_views (bool): Whether to try to create resource views that point to the datastore
+
+        Returns:
+            None
+        """
+        package = deepcopy(self.data)
+        if self.resources:
+            package['resources'] = self._convert_hdxobjects(self.resources)
+
+        data = {'package': package, 'create_datastore_views': create_datastore_views}
+        self._write_to_hdx('create_default_views', data, 'package')
