@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 """Configuration for HDX"""
+import requests
 import six
 
 from hdx.utilities.email import Email
@@ -13,13 +14,13 @@ else:
 
 import logging
 from base64 import b64decode
-from os.path import expanduser, join
-from typing import Optional
+from os.path import expanduser, join, isfile
+from typing import Optional, Any, Dict
 
 import ckanapi
 
 from hdx.utilities.dictandlist import merge_two_dictionaries
-from hdx.utilities.loader import load_yaml, load_json, load_file_to_str
+from hdx.utilities.loader import load_yaml, load_json
 from hdx.utilities.path import script_dir_plus_file
 
 logger = logging.getLogger(__name__)
@@ -37,17 +38,22 @@ class Configuration(UserDict, object):
         hdx_site (Optional[str]): HDX site to use eg. prod, test. Defaults to test.
         hdx_read_only (bool): Whether to access HDX in read only mode. Defaults to False.
         hdx_key (Optional[str]): Your HDX key. Ignored if hdx_read_only = True.
-        hdx_key_file (Optional[str]): Path to HDX key file. Ignored if hdx_read_only = True or hdx_key supplied. Defaults to ~/.hdxkey.
-        hdx_config_dict (dict): HDX configuration dictionary OR
+        hdx_config_dict (dict): HDX configuration dictionary to use instead of above 3 parameters OR
         hdx_config_json (str): Path to JSON HDX configuration OR
-        hdx_config_yaml (str): Path to YAML HDX configuration. Defaults to library's internal hdx_configuration.yml.
+        hdx_config_yaml (str): Path to YAML HDX configuration
         project_config_dict (dict): Project configuration dictionary OR
         project_config_json (str): Path to JSON Project configuration OR
         project_config_yaml (str): Path to YAML Project configuration
+        hdx_base_config_dict (dict): HDX base configuration dictionary OR
+        hdx_base_config_json (str): Path to JSON HDX base configuration OR
+        hdx_base_config_yaml (str): Path to YAML HDX base configuration. Defaults to library's internal hdx_base_configuration.yml.
     """
 
     _configuration = None
-    default_hdx_key_file = join(expanduser('~'), '.hdxkey')
+    home_folder = expanduser('~')
+    default_hdx_base_config_yaml = script_dir_plus_file('hdx_base_configuration.yml', ConfigurationError)
+    default_hdx_config_yaml = join(home_folder, '.hdx_configuration.yml')
+    default_user_agent_config_yaml = join(home_folder, '.useragent.yml')
 
     def __init__(self, **kwargs):
         # type: (Any) -> None
@@ -56,6 +62,31 @@ class Configuration(UserDict, object):
         self._remoteckan = None
         self._emailer = None
 
+        hdx_base_config_found = False
+        hdx_base_config_dict = kwargs.get('hdx_base_config_dict', None)
+        if hdx_base_config_dict:
+            hdx_base_config_found = True
+            logger.info('Loading HDX base configuration from dictionary')
+
+        hdx_base_config_json = kwargs.get('hdx_base_config_json', '')
+        if hdx_base_config_json:
+            if hdx_base_config_found:
+                raise ConfigurationError('More than one HDX base configuration given!')
+            hdx_base_config_found = True
+            logger.info('Loading HDX base configuration from: %s' % hdx_base_config_json)
+            hdx_base_config_dict = load_json(hdx_base_config_json)
+
+        hdx_base_config_yaml = kwargs.get('hdx_base_config_yaml', '')
+        if hdx_base_config_found:
+            if hdx_base_config_yaml:
+                raise ConfigurationError('More than one HDX base configuration given!')
+        else:
+            if not hdx_base_config_yaml:
+                hdx_base_config_yaml = Configuration.default_hdx_base_config_yaml
+                logger.info('No HDX base configuration parameter. Using default base configuration file: %s.' % hdx_base_config_yaml)
+            logger.info('Loading HDX base configuration from: %s' % hdx_base_config_yaml)
+            hdx_base_config_dict = load_yaml(hdx_base_config_yaml)
+
         hdx_config_found = False
         hdx_config_dict = kwargs.get('hdx_config_dict', None)
         if hdx_config_dict:
@@ -76,10 +107,18 @@ class Configuration(UserDict, object):
                 raise ConfigurationError('More than one HDX configuration given!')
         else:
             if not hdx_config_yaml:
-                logger.info('No HDX configuration parameter. Using default.')
-                hdx_config_yaml = script_dir_plus_file('hdx_configuration.yml', Configuration)
-            logger.info('Loading HDX configuration from: %s' % hdx_config_yaml)
-            hdx_config_dict = load_yaml(hdx_config_yaml)
+                hdx_config_yaml = Configuration.default_hdx_config_yaml
+                if isfile(hdx_config_yaml):
+                    logger.info('No HDX configuration parameter. Using default configuration file: %s.' % hdx_config_yaml)
+                else:
+                    logger.info('No HDX configuration parameter and no configuration file at default path: %s.' % hdx_config_yaml)
+                    hdx_config_yaml = None
+                    hdx_config_dict = dict()
+            if hdx_config_yaml:
+                logger.info('Loading HDX configuration from: %s' % hdx_config_yaml)
+                hdx_config_dict = load_yaml(hdx_config_yaml)
+
+        self.data = merge_two_dictionaries(hdx_base_config_dict, hdx_config_dict)
 
         project_config_found = False
         project_config_dict = kwargs.get('project_config_dict', None)
@@ -106,23 +145,22 @@ class Configuration(UserDict, object):
             else:
                 project_config_dict = dict()
 
-        self.data = merge_two_dictionaries(hdx_config_dict, project_config_dict)
+        self.data = merge_two_dictionaries(hdx_base_config_dict, project_config_dict)
 
-        self.hdx_read_only = kwargs.get('hdx_read_only', False)
-        if not self.hdx_read_only:
-            if 'hdx_key' in kwargs:
-                self.data['api_key'] = kwargs.get('hdx_key')
-            else:
-                hdx_key_file = kwargs.get('hdx_key_file', None)
-                if not hdx_key_file:
-                    logger.info('No HDX key or key file given. Using default key file path.')
-                    hdx_key_file = Configuration.default_hdx_key_file
-                self.data['api_key'] = self.load_api_key(hdx_key_file)
-
-        self.hdx_site = 'hdx_%s_site' % kwargs.get('hdx_site', 'test')
+        self.hdx_read_only = kwargs.get('hdx_read_only', self.data.get('hdx_read_only', False))
+        logger.info('Read only access to HDX: %s' % str(self.hdx_read_only))
+        self.hdx_key = kwargs.get('hdx_key', self.data.get('hdx_key'))
+        if not self.hdx_key and not self.hdx_read_only:
+            raise ConfigurationError('No HDX API key supplied as a parameter or in configuration!')
+        self.hdx_site = 'hdx_%s_site' % kwargs.get('hdx_site', self.data.get('hdx_site', 'test'))
         if self.hdx_site not in self.data:
             raise ConfigurationError('%s not defined in configuration!' % self.hdx_site)
 
+    @staticmethod
+    def get_version():
+        version_file = open(script_dir_plus_file('version.txt', Configuration))
+        return version_file.read().strip()
+
     def set_read_only(self, read_only=True):
         # type: (bool) -> None
         """
@@ -147,7 +185,7 @@ class Configuration(UserDict, object):
             None
 
         """
-        self.data['api_key'] = apikey
+        self.hdx_key = apikey
 
     def get_api_key(self):
         # type: () -> Optional[str]
@@ -160,7 +198,7 @@ class Configuration(UserDict, object):
         """
         if self.hdx_read_only:
             return None
-        return self.data['api_key']
+        return self.hdx_key
 
     def get_hdx_site_url(self):
         # type: () -> str
@@ -222,30 +260,42 @@ class Configuration(UserDict, object):
         kwargs['apikey'] = apikey
         return self.remoteckan().call_action(*args, **kwargs)
 
-    def create_remoteckan(self, session=get_session(method_whitelist=frozenset(['HEAD', 'TRACE', 'GET', 'POST', 'PUT',
-                                                                                'OPTIONS', 'DELETE']))):
-        # type: () -> ckanapi.RemoteCKAN
+    @classmethod
+    def create_remoteckan(cls, site_url, user_agent=None, user_agent_config_yaml=None, user_agent_lookup=None,
+                          session=get_session(method_whitelist=frozenset(['HEAD', 'TRACE', 'GET', 'POST', 'PUT',
+                                                                          'OPTIONS', 'DELETE'])), **kwargs):
+        # type: (str, Optional[str], Optional[str], Optional[str], requests.Session, Any) -> ckanapi.RemoteCKAN
         """
         Create remote CKAN instance from configuration
 
         Args:
-            session: requests Session object to use. Defaults to calling hdx.utilities.session.get_session()
+            site_url (str): Site url.
+            user_agent (Optional[str]): User agent string. HDXPythonLibrary/X.X.X- is prefixed.
+            user_agent_config_yaml (Optional[str]): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
+            user_agent_lookup (Optional[str]): Lookup key for YAML. Ignored if user_agent supplied.
+            session (requests.Session): requests Session object to use. Defaults to calling hdx.utilities.session.get_session()
 
         Returns:
             ckanapi.RemoteCKAN: Remote CKAN instance
 
         """
-        version_file = open(script_dir_plus_file('version.txt', Configuration))
-        version = version_file.read().strip()
-        return ckanapi.RemoteCKAN(self.get_hdx_site_url(), session=session,
-                                  user_agent='HDXPythonLibrary/%s' % version)
+        prefix = 'HDXPythonLibrary/%s' % Configuration.get_version()
+        if not user_agent:
+            ua = Configuration.load_user_agent(prefix, user_agent_config_yaml, user_agent_lookup)
+        else:
+            ua = cls._construct_user_agent(kwargs, prefix, user_agent)
+        return ckanapi.RemoteCKAN(site_url, user_agent=ua, session=session)
 
-    def setup_remoteckan(self, remoteckan=None):
-        # type: (Optional[ckanapi.RemoteCKAN]) -> None
+    def setup_remoteckan(self, user_agent=None, user_agent_config_yaml=None, user_agent_lookup=None,
+                         remoteckan=None, **kwargs):
+        # type: (Optional[str], Optional[str], Optional[str], Optional[ckanapi.RemoteCKAN], Any) -> None
         """
         Set up remote CKAN from provided CKAN or by creating from configuration
 
         Args:
+            user_agent (Optional[str]): User agent string. HDXPythonLibrary/X.X.X- is prefixed. Must be supplied if remoteckan is not.
+            user_agent_config_yaml (Optional[str]): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
+            user_agent_lookup (Optional[str]): Lookup key for YAML. Ignored if user_agent supplied.
             remoteckan (Optional[ckanapi.RemoteCKAN]): CKAN instance. Defaults to setting one up from configuration.
 
         Returns:
@@ -253,7 +303,8 @@ class Configuration(UserDict, object):
 
         """
         if remoteckan is None:
-            self._remoteckan = self.create_remoteckan()
+            self._remoteckan = self.create_remoteckan(self.get_hdx_site_url(), user_agent, user_agent_config_yaml,
+                                                      user_agent_lookup, **kwargs)
         else:
             self._remoteckan = remoteckan
 
@@ -294,21 +345,60 @@ class Configuration(UserDict, object):
         self._emailer = Email(**kwargs)
 
     @staticmethod
-    def load_api_key(path):
-        # type: (str) -> str
+    def _construct_user_agent(configdict, prefix, ua):
+        # type: (Dict, str, str) -> str
         """
-        Load HDX api key
+        Construct user agent
 
         Args:
-            path (str): Path to HDX key
+            configdict (str): Additional configuration for user agent
+            prefix (str): Text to put at start of user agent
+            ua (str): Custom user agent text
 
         Returns:
-            str: HDX api key
+            str: Full user agent string
 
         """
-        logger.info('Loading HDX api key from: %s' % path)
-        apikey = load_file_to_str(path)
-        return apikey
+        if not ua:
+            raise ConfigurationError("user_agent should be supplied in a YAML config file or directly as a parameter. It can be your project's name for example.")
+        preprefix = configdict.get('preprefix')
+        if preprefix:
+            user_agent = '%s:' % preprefix
+        else:
+            user_agent = ''
+        if prefix:
+            user_agent = '%s%s-' % (user_agent, prefix)
+        user_agent = '%s%s' % (user_agent, ua)
+        return user_agent
+
+    @classmethod
+    def load_user_agent(cls, prefix, user_agent_config_yaml, user_agent_lookup=None):
+        # type: (str, str, Optional[str]) -> str
+        """
+        Load user agent YAML file
+
+        Args:
+            prefix (str): Text to put at start of user agent
+            user_agent_config_yaml (str): Path to user agent YAML file
+            user_agent_lookup (Optional[str]): Lookup key for YAML. Ignored if user_agent supplied.
+
+        Returns:
+            str: user agent
+
+        """
+        if not user_agent_config_yaml:
+            user_agent_config_yaml = Configuration.default_user_agent_config_yaml
+            logger.info('No user agent or user agent config file given. Using default user agent config file: %s.' % user_agent_config_yaml)
+        if not isfile(user_agent_config_yaml):
+            raise ConfigurationError("User_agent should be supplied in a YAML config file or directly as a parameter. It can be your project's name for example.")
+        logger.info('Loading user agent config from: %s' % user_agent_config_yaml)
+        user_agent_config_dict = load_yaml(user_agent_config_yaml)
+        if user_agent_lookup:
+            user_agent_config_dict = user_agent_config_dict.get(user_agent_lookup)
+        if not user_agent_config_dict:
+            raise ConfigurationError("No user agent information read from: %s" % user_agent_config_yaml)
+        ua = user_agent_config_dict.get('user_agent')
+        return cls._construct_user_agent(user_agent_config_dict, prefix, ua)
 
     @classmethod
     def read(cls):
@@ -336,13 +426,15 @@ class Configuration(UserDict, object):
             hdx_site (Optional[str]): HDX site to use eg. prod, test. Defaults to test.
             hdx_read_only (bool): Whether to access HDX in read only mode. Defaults to False.
             hdx_key (Optional[str]): Your HDX key. Ignored if hdx_read_only = True.
-            hdx_key_file (Optional[str]): Path to HDX key file. Ignored if hdx_read_only = True or hdx_key supplied. Defaults to ~/.hdxkey.
-            hdx_config_dict (dict): HDX configuration dictionary OR
+            hdx_config_dict (dict): HDX configuration dictionary to use instead of above 3 parameters OR
             hdx_config_json (str): Path to JSON HDX configuration OR
-            hdx_config_yaml (str): Path to YAML HDX configuration. Defaults to library's internal hdx_configuration.yml.
+            hdx_config_yaml (str): Path to YAML HDX configuration
             project_config_dict (dict): Project configuration dictionary OR
             project_config_json (str): Path to JSON Project configuration OR
             project_config_yaml (str): Path to YAML Project configuration
+            hdx_base_config_dict (dict): HDX base configuration dictionary OR
+            hdx_base_config_json (str): Path to JSON HDX base configuration OR
+            hdx_base_config_yaml (str): Path to YAML HDX base configuration. Defaults to library's internal hdx_base_configuration.yml.
 
         Returns:
             None
@@ -354,54 +446,66 @@ class Configuration(UserDict, object):
             cls._configuration = configuration
 
     @classmethod
-    def _create(cls, configuration=None, remoteckan=None, **kwargs):
-        # type: (Optional['Configuration'], Optional[ckanapi.RemoteCKAN], Any) -> str
+    def _create(cls, configuration=None, user_agent=None, user_agent_config_yaml=None, user_agent_lookup=None,
+                remoteckan=None, **kwargs):
+        # type: (Optional['Configuration'], Optional[str], Optional[str], Optional[str], Optional[ckanapi.RemoteCKAN], Any) -> str
         """
         Create HDX configuration
 
         Args:
             configuration (Optional[Configuration]): Configuration instance. Defaults to setting one up from passed arguments.
+            user_agent (Optional[str]): User agent string. HDXPythonLibrary/X.X.X- is prefixed. Must be supplied if remoteckan is not.
+            user_agent_config_yaml (Optional[str]): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
+            user_agent_lookup (Optional[str]): Lookup key for YAML. Ignored if user_agent supplied.
             remoteckan (Optional[ckanapi.RemoteCKAN]): CKAN instance. Defaults to setting one up from configuration.
             **kwargs: See below
             hdx_site (Optional[str]): HDX site to use eg. prod, test. Defaults to test.
             hdx_read_only (bool): Whether to access HDX in read only mode. Defaults to False.
             hdx_key (Optional[str]): Your HDX key. Ignored if hdx_read_only = True.
-            hdx_key_file (Optional[str]): Path to HDX key file. Ignored if hdx_read_only = True or hdx_key supplied. Defaults to ~/.hdxkey.
-            hdx_config_dict (dict): HDX configuration dictionary OR
+            hdx_config_dict (dict): HDX configuration dictionary to use instead of above 3 parameters OR
             hdx_config_json (str): Path to JSON HDX configuration OR
-            hdx_config_yaml (str): Path to YAML HDX configuration. Defaults to library's internal hdx_configuration.yml.
+            hdx_config_yaml (str): Path to YAML HDX configuration
             project_config_dict (dict): Project configuration dictionary OR
             project_config_json (str): Path to JSON Project configuration OR
             project_config_yaml (str): Path to YAML Project configuration
+            hdx_base_config_dict (dict): HDX base configuration dictionary OR
+            hdx_base_config_json (str): Path to JSON HDX base configuration OR
+            hdx_base_config_yaml (str): Path to YAML HDX base configuration. Defaults to library's internal hdx_base_configuration.yml.
 
         Returns:
             str: HDX site url
 
         """
         cls.setup(configuration, **kwargs)
-        cls._configuration.setup_remoteckan(remoteckan)
+        cls._configuration.setup_remoteckan(user_agent, user_agent_config_yaml, user_agent_lookup, remoteckan, **kwargs)
         return cls._configuration.get_hdx_site_url()
 
     @classmethod
-    def create(cls, configuration=None, remoteckan=None, **kwargs):
-        # type: (Optional['Configuration'], Optional[ckanapi.RemoteCKAN], Any) -> str
+    def create(cls, configuration=None, user_agent=None, user_agent_config_yaml=None, user_agent_lookup=None,
+               remoteckan=None, **kwargs):
+        # type: (Optional['Configuration'], Optional[str], Optional[str], Optional[str], Optional[ckanapi.RemoteCKAN], Any) -> str
         """
         Create HDX configuration. Can only be called once (will raise an error if called more than once).
 
         Args:
             configuration (Optional[Configuration]): Configuration instance. Defaults to setting one up from passed arguments.
+            user_agent (Optional[str]): User agent string. HDXPythonLibrary/X.X.X- is prefixed. Must be supplied if remoteckan is not.
+            user_agent_config_yaml (Optional[str]): Path to YAML user agent configuration. Ignored if user_agent supplied. Defaults to ~/.useragent.yml.
+            user_agent_lookup (Optional[str]): Lookup key for YAML. Ignored if user_agent supplied.
             remoteckan (Optional[ckanapi.RemoteCKAN]): CKAN instance. Defaults to setting one up from configuration.
             **kwargs: See below
-            hdx_site (Optional[str]): HDX site to use eg. prod, test. Defaults to test.
+            hdx_site (str): HDX site to use eg. prod, test.
             hdx_read_only (bool): Whether to access HDX in read only mode. Defaults to False.
             hdx_key (Optional[str]): Your HDX key. Ignored if hdx_read_only = True.
-            hdx_key_file (Optional[str]): Path to HDX key file. Ignored if hdx_read_only = True or hdx_key supplied. Defaults to ~/.hdxkey.
-            hdx_config_dict (dict): HDX configuration dictionary OR
+            hdx_config_dict (dict): HDX configuration dictionary to use instead of above 3 parameters OR
             hdx_config_json (str): Path to JSON HDX configuration OR
-            hdx_config_yaml (str): Path to YAML HDX configuration. Defaults to library's internal hdx_configuration.yml.
+            hdx_config_yaml (str): Path to YAML HDX configuration
             project_config_dict (dict): Project configuration dictionary OR
             project_config_json (str): Path to JSON Project configuration OR
             project_config_yaml (str): Path to YAML Project configuration
+            hdx_base_config_dict (dict): HDX base configuration dictionary OR
+            hdx_base_config_json (str): Path to JSON HDX base configuration OR
+            hdx_base_config_yaml (str): Path to YAML HDX base configuration. Defaults to library's internal hdx_base_configuration.yml.
 
         Returns:
             str: HDX site url
@@ -409,7 +513,9 @@ class Configuration(UserDict, object):
         """
         if cls._configuration is not None:
             raise ConfigurationError('Configuration already created!')
-        return cls._create(configuration=configuration, remoteckan=remoteckan, **kwargs)
+        return cls._create(configuration=configuration, user_agent=user_agent,
+                           user_agent_config_yaml=user_agent_config_yaml, user_agent_lookup=user_agent_lookup,
+                           remoteckan=remoteckan, **kwargs)
 
     @classmethod
     def delete(cls):
